@@ -1,12 +1,12 @@
+use ruff_python_semantic::SemanticModel;
 use rustc_hash::FxHashSet;
 
-use ruff_diagnostics::Diagnostic;
-use ruff_diagnostics::Violation;
-use ruff_macros::{derive_message_formats, violation};
+use ruff_macros::{ViolationMetadata, derive_message_formats};
 use ruff_python_ast::comparable::ComparableExpr;
-use ruff_python_ast::{self as ast, Expr, PySourceType, Stmt};
+use ruff_python_ast::{self as ast, Expr, ExprCall, Stmt};
 use ruff_text_size::Ranged;
 
+use crate::Violation;
 use crate::checkers::ast::Checker;
 
 /// ## What it does
@@ -40,8 +40,8 @@ use crate::checkers::ast::Checker;
 ///
 /// ## References
 /// - [Python documentation: `enum.Enum`](https://docs.python.org/3/library/enum.html#enum.Enum)
-#[violation]
-pub struct NonUniqueEnums {
+#[derive(ViolationMetadata)]
+pub(crate) struct NonUniqueEnums {
     value: String,
 }
 
@@ -54,14 +54,15 @@ impl Violation for NonUniqueEnums {
 }
 
 /// PIE796
-pub(crate) fn non_unique_enums(checker: &mut Checker, parent: &Stmt, body: &[Stmt]) {
+pub(crate) fn non_unique_enums(checker: &Checker, parent: &Stmt, body: &[Stmt]) {
+    let semantic = checker.semantic();
+
     let Stmt::ClassDef(parent) = parent else {
         return;
     };
 
     if !parent.bases().iter().any(|expr| {
-        checker
-            .semantic()
+        semantic
             .resolve_qualified_name(expr)
             .is_some_and(|qualified_name| matches!(qualified_name.segments(), ["enum", "Enum"]))
     }) {
@@ -74,32 +75,59 @@ pub(crate) fn non_unique_enums(checker: &mut Checker, parent: &Stmt, body: &[Stm
             continue;
         };
 
-        if let Expr::Call(ast::ExprCall { func, .. }) = value.as_ref() {
-            if checker
-                .semantic()
-                .resolve_qualified_name(func)
-                .is_some_and(|qualified_name| matches!(qualified_name.segments(), ["enum", "auto"]))
-            {
+        if is_call_to_enum_auto(semantic, value) {
+            continue;
+        } else if let Expr::Tuple(ast::ExprTuple { elts, .. }) = value.as_ref() {
+            if elts.iter().any(|elt| is_call_to_enum_auto(semantic, elt)) {
                 continue;
             }
         }
 
-        let comparable = ComparableExpr::from(value);
-
-        if checker.source_type == PySourceType::Stub
-            && comparable == ComparableExpr::EllipsisLiteral
-        {
+        if checker.source_type.is_stub() && member_has_unknown_value(semantic, value) {
             continue;
         }
 
+        let comparable = ComparableExpr::from(value);
+
         if !seen_targets.insert(comparable) {
-            let diagnostic = Diagnostic::new(
+            checker.report_diagnostic(
                 NonUniqueEnums {
                     value: checker.generator().expr(value),
                 },
                 stmt.range(),
             );
-            checker.diagnostics.push(diagnostic);
         }
     }
 }
+
+fn is_call_to_enum_auto(semantic: &SemanticModel, expr: &Expr) -> bool {
+    expr.as_call_expr().is_some_and(|call| {
+        semantic
+            .resolve_qualified_name(&call.func)
+            .is_some_and(|qualified_name| matches!(qualified_name.segments(), ["enum", "auto"]))
+    })
+}
+
+/// Whether the value is a bare ellipsis literal (`A = ...`)
+/// or a casted one (`A = cast(SomeType, ...)`).
+fn member_has_unknown_value(semantic: &SemanticModel, expr: &Expr) -> bool {
+    match expr {
+        Expr::EllipsisLiteral(_) => true,
+
+        Expr::Call(ExprCall {
+            func, arguments, ..
+        }) => {
+            if !semantic.match_typing_expr(func, "cast") {
+                return false;
+            }
+
+            if !arguments.keywords.is_empty() {
+                return false;
+            }
+
+            matches!(arguments.args.as_ref(), [_, Expr::EllipsisLiteral(_)])
+        }
+
+        _ => false,
+    }
+}
