@@ -16,6 +16,9 @@
 //! have the same shape in that they evaluate to the same value.
 
 use crate as ast;
+use crate::{Expr, Number};
+use std::borrow::Cow;
+use std::hash::Hash;
 
 #[derive(Debug, PartialEq, Eq, Hash, Copy, Clone)]
 pub enum ComparableBoolOp {
@@ -223,7 +226,6 @@ pub struct PatternMatchOr<'a> {
     patterns: Vec<ComparablePattern<'a>>,
 }
 
-#[allow(clippy::enum_variant_names)]
 #[derive(Debug, PartialEq, Eq, Hash)]
 pub enum ComparablePattern<'a> {
     MatchValue(PatternMatchValue<'a>),
@@ -510,40 +512,61 @@ impl<'a> From<&'a ast::ExceptHandler> for ComparableExceptHandler<'a> {
 }
 
 #[derive(Debug, PartialEq, Eq, Hash)]
-pub enum ComparableFStringElement<'a> {
-    Literal(&'a str),
-    FStringExpressionElement(FStringExpressionElement<'a>),
+pub enum ComparableInterpolatedStringElement<'a> {
+    Literal(Cow<'a, str>),
+    InterpolatedElement(InterpolatedElement<'a>),
 }
 
 #[derive(Debug, PartialEq, Eq, Hash)]
-pub struct FStringExpressionElement<'a> {
+pub struct InterpolatedElement<'a> {
     expression: ComparableExpr<'a>,
     debug_text: Option<&'a ast::DebugText>,
     conversion: ast::ConversionFlag,
-    format_spec: Option<Vec<ComparableFStringElement<'a>>>,
+    format_spec: Option<Vec<ComparableInterpolatedStringElement<'a>>>,
 }
 
-impl<'a> From<&'a ast::FStringElement> for ComparableFStringElement<'a> {
-    fn from(fstring_element: &'a ast::FStringElement) -> Self {
-        match fstring_element {
-            ast::FStringElement::Literal(ast::FStringLiteralElement { value, .. }) => {
-                Self::Literal(value)
-            }
-            ast::FStringElement::Expression(formatted_value) => {
-                Self::FStringExpressionElement(FStringExpressionElement {
-                    expression: (&formatted_value.expression).into(),
-                    debug_text: formatted_value.debug_text.as_ref(),
-                    conversion: formatted_value.conversion,
-                    format_spec: formatted_value
-                        .format_spec
-                        .as_ref()
-                        .map(|spec| spec.elements.iter().map(Into::into).collect()),
-                })
+impl<'a> From<&'a ast::InterpolatedStringElement> for ComparableInterpolatedStringElement<'a> {
+    fn from(interpolated_string_element: &'a ast::InterpolatedStringElement) -> Self {
+        match interpolated_string_element {
+            ast::InterpolatedStringElement::Literal(ast::InterpolatedStringLiteralElement {
+                value,
+                ..
+            }) => Self::Literal(value.as_ref().into()),
+            ast::InterpolatedStringElement::Interpolation(formatted_value) => {
+                formatted_value.into()
             }
         }
     }
 }
 
+impl<'a> From<&'a ast::InterpolatedElement> for InterpolatedElement<'a> {
+    fn from(interpolated_element: &'a ast::InterpolatedElement) -> Self {
+        let ast::InterpolatedElement {
+            expression,
+            debug_text,
+            conversion,
+            format_spec,
+            range: _,
+            node_index: _,
+        } = interpolated_element;
+
+        Self {
+            expression: (expression).into(),
+            debug_text: debug_text.as_ref(),
+            conversion: *conversion,
+            format_spec: format_spec
+                .as_ref()
+                .map(|spec| spec.elements.iter().map(Into::into).collect()),
+        }
+    }
+}
+
+impl<'a> From<&'a ast::InterpolatedElement> for ComparableInterpolatedStringElement<'a> {
+    fn from(interpolated_element: &'a ast::InterpolatedElement) -> Self {
+        Self::InterpolatedElement(interpolated_element.into())
+    }
+}
+
 #[derive(Debug, PartialEq, Eq, Hash)]
 pub struct ComparableElifElseClause<'a> {
     test: Option<ComparableExpr<'a>>,
@@ -554,6 +577,7 @@ impl<'a> From<&'a ast::ElifElseClause> for ComparableElifElseClause<'a> {
     fn from(elif_else_clause: &'a ast::ElifElseClause) -> Self {
         let ast::ElifElseClause {
             range: _,
+            node_index: _,
             test,
             body,
         } = elif_else_clause;
@@ -597,28 +621,162 @@ impl<'a> From<ast::LiteralExpressionRef<'a>> for ComparableLiteral<'a> {
 
 #[derive(Debug, PartialEq, Eq, Hash)]
 pub struct ComparableFString<'a> {
-    elements: Vec<ComparableFStringElement<'a>>,
+    elements: Box<[ComparableInterpolatedStringElement<'a>]>,
 }
 
-impl<'a> From<&'a ast::FString> for ComparableFString<'a> {
-    fn from(fstring: &'a ast::FString) -> Self {
+impl<'a> From<&'a ast::FStringValue> for ComparableFString<'a> {
+    // The approach below is somewhat complicated, so it may
+    // require some justification.
+    //
+    // Suppose given an f-string of the form
+    // `f"{foo!r} one" " and two " f" and three {bar!s}"`
+    // This decomposes as:
+    // - An `FStringPart::FString`, `f"{foo!r} one"` with elements
+    //      - `FStringElement::Expression` encoding `{foo!r}`
+    //      - `FStringElement::Literal` encoding " one"
+    // - An `FStringPart::Literal` capturing `" and two "`
+    // - An `FStringPart::FString`, `f" and three {bar!s}"` with elements
+    //      - `FStringElement::Literal` encoding " and three "
+    //      - `FStringElement::Expression` encoding `{bar!s}`
+    //
+    // We would like to extract from this a vector of (comparable) f-string
+    // _elements_ which alternate between expression elements and literal
+    // elements. In order to do so, we need to concatenate adjacent string
+    // literals. String literals may be separated for two reasons: either
+    // they appear in adjacent string literal parts, or else a string literal
+    // part is adjacent to a string literal _element_ inside of an f-string part.
+    fn from(value: &'a ast::FStringValue) -> Self {
+        #[derive(Default)]
+        struct Collector<'a> {
+            elements: Vec<ComparableInterpolatedStringElement<'a>>,
+        }
+
+        impl<'a> Collector<'a> {
+            // The logic for concatenating adjacent string literals
+            // occurs here, implicitly: when we encounter a sequence
+            // of string literals, the first gets pushed to the
+            // `elements` vector, while subsequent strings
+            // are concatenated onto this top string.
+            fn push_literal(&mut self, literal: &'a str) {
+                if let Some(ComparableInterpolatedStringElement::Literal(existing_literal)) =
+                    self.elements.last_mut()
+                {
+                    existing_literal.to_mut().push_str(literal);
+                } else {
+                    self.elements
+                        .push(ComparableInterpolatedStringElement::Literal(literal.into()));
+                }
+            }
+
+            fn push_expression(&mut self, expression: &'a ast::InterpolatedElement) {
+                self.elements.push(expression.into());
+            }
+        }
+
+        let mut collector = Collector::default();
+
+        for part in value {
+            match part {
+                ast::FStringPart::Literal(string_literal) => {
+                    collector.push_literal(&string_literal.value);
+                }
+                ast::FStringPart::FString(fstring) => {
+                    for element in &fstring.elements {
+                        match element {
+                            ast::InterpolatedStringElement::Literal(literal) => {
+                                collector.push_literal(&literal.value);
+                            }
+                            ast::InterpolatedStringElement::Interpolation(expression) => {
+                                collector.push_expression(expression);
+                            }
+                        }
+                    }
+                }
+            }
+        }
+
         Self {
-            elements: fstring.elements.iter().map(Into::into).collect(),
+            elements: collector.elements.into_boxed_slice(),
         }
     }
 }
 
 #[derive(Debug, PartialEq, Eq, Hash)]
-pub enum ComparableFStringPart<'a> {
-    Literal(ComparableStringLiteral<'a>),
-    FString(ComparableFString<'a>),
+pub struct ComparableTString<'a> {
+    strings: Box<[ComparableInterpolatedStringElement<'a>]>,
+    interpolations: Box<[InterpolatedElement<'a>]>,
 }
 
-impl<'a> From<&'a ast::FStringPart> for ComparableFStringPart<'a> {
-    fn from(f_string_part: &'a ast::FStringPart) -> Self {
-        match f_string_part {
-            ast::FStringPart::Literal(string_literal) => Self::Literal(string_literal.into()),
-            ast::FStringPart::FString(f_string) => Self::FString(f_string.into()),
+impl<'a> From<&'a ast::TStringValue> for ComparableTString<'a> {
+    // We model a [`ComparableTString`] on the actual
+    // [CPython implementation] of a `string.templatelib.Template` object.
+    //
+    // As in CPython, we must be careful to ensure that the length
+    // of `strings` is always one more than the length of `interpolations` -
+    // that way we can recover the original reading order by interleaving
+    // starting with `strings`. This is how we can tell the
+    // difference between, e.g. `t"{foo}bar"` and `t"bar{foo}"`.
+    //
+    // - [CPython implementation](https://github.com/python/cpython/blob/c91ad5da9d92eac4718e4da8d53689c3cc24535e/Python/codegen.c#L4052-L4103)
+    fn from(value: &'a ast::TStringValue) -> Self {
+        struct Collector<'a> {
+            strings: Vec<ComparableInterpolatedStringElement<'a>>,
+            interpolations: Vec<InterpolatedElement<'a>>,
+        }
+
+        impl Default for Collector<'_> {
+            fn default() -> Self {
+                Self {
+                    strings: vec![ComparableInterpolatedStringElement::Literal("".into())],
+                    interpolations: vec![],
+                }
+            }
+        }
+
+        impl<'a> Collector<'a> {
+            // The logic for concatenating adjacent string literals
+            // occurs here, implicitly: when we encounter a sequence
+            // of string literals, the first gets pushed to the
+            // `strings` vector, while subsequent strings
+            // are concatenated onto this top string.
+            fn push_literal(&mut self, literal: &'a str) {
+                if let Some(ComparableInterpolatedStringElement::Literal(existing_literal)) =
+                    self.strings.last_mut()
+                {
+                    existing_literal.to_mut().push_str(literal);
+                } else {
+                    self.strings
+                        .push(ComparableInterpolatedStringElement::Literal(literal.into()));
+                }
+            }
+
+            fn start_new_literal(&mut self) {
+                self.strings
+                    .push(ComparableInterpolatedStringElement::Literal("".into()));
+            }
+
+            fn push_tstring_interpolation(&mut self, expression: &'a ast::InterpolatedElement) {
+                self.interpolations.push(expression.into());
+                self.start_new_literal();
+            }
+        }
+
+        let mut collector = Collector::default();
+
+        for element in value.elements() {
+            match element {
+                ast::InterpolatedStringElement::Literal(literal) => {
+                    collector.push_literal(&literal.value);
+                }
+                ast::InterpolatedStringElement::Interpolation(interpolation) => {
+                    collector.push_tstring_interpolation(interpolation);
+                }
+            }
+        }
+
+        Self {
+            strings: collector.strings.into_boxed_slice(),
+            interpolations: collector.interpolations.into_boxed_slice(),
         }
     }
 }
@@ -638,13 +796,13 @@ impl<'a> From<&'a ast::StringLiteral> for ComparableStringLiteral<'a> {
 
 #[derive(Debug, PartialEq, Eq, Hash)]
 pub struct ComparableBytesLiteral<'a> {
-    value: &'a [u8],
+    value: Cow<'a, [u8]>,
 }
 
 impl<'a> From<&'a ast::BytesLiteral> for ComparableBytesLiteral<'a> {
     fn from(bytes_literal: &'a ast::BytesLiteral) -> Self {
         Self {
-            value: &bytes_literal.value,
+            value: Cow::Borrowed(&bytes_literal.value),
         }
     }
 }
@@ -766,26 +924,31 @@ pub struct ExprCall<'a> {
 }
 
 #[derive(Debug, PartialEq, Eq, Hash)]
-pub struct ExprFStringExpressionElement<'a> {
+pub struct ExprInterpolatedElement<'a> {
     value: Box<ComparableExpr<'a>>,
     debug_text: Option<&'a ast::DebugText>,
     conversion: ast::ConversionFlag,
-    format_spec: Vec<ComparableFStringElement<'a>>,
+    format_spec: Vec<ComparableInterpolatedStringElement<'a>>,
 }
 
 #[derive(Debug, PartialEq, Eq, Hash)]
 pub struct ExprFString<'a> {
-    parts: Vec<ComparableFStringPart<'a>>,
+    value: ComparableFString<'a>,
+}
+
+#[derive(Debug, PartialEq, Eq, Hash)]
+pub struct ExprTString<'a> {
+    value: ComparableTString<'a>,
 }
 
 #[derive(Debug, PartialEq, Eq, Hash)]
 pub struct ExprStringLiteral<'a> {
-    parts: Vec<ComparableStringLiteral<'a>>,
+    value: ComparableStringLiteral<'a>,
 }
 
 #[derive(Debug, PartialEq, Eq, Hash)]
 pub struct ExprBytesLiteral<'a> {
-    parts: Vec<ComparableBytesLiteral<'a>>,
+    value: ComparableBytesLiteral<'a>,
 }
 
 #[derive(Debug, PartialEq, Eq, Hash)]
@@ -794,8 +957,8 @@ pub struct ExprNumberLiteral<'a> {
 }
 
 #[derive(Debug, PartialEq, Eq, Hash)]
-pub struct ExprBoolLiteral<'a> {
-    value: &'a bool,
+pub struct ExprBoolLiteral {
+    value: bool,
 }
 
 #[derive(Debug, PartialEq, Eq, Hash)]
@@ -862,12 +1025,14 @@ pub enum ComparableExpr<'a> {
     YieldFrom(ExprYieldFrom<'a>),
     Compare(ExprCompare<'a>),
     Call(ExprCall<'a>),
-    FStringExpressionElement(ExprFStringExpressionElement<'a>),
+    FStringExpressionElement(ExprInterpolatedElement<'a>),
     FString(ExprFString<'a>),
+    TStringInterpolationElement(ExprInterpolatedElement<'a>),
+    TString(ExprTString<'a>),
     StringLiteral(ExprStringLiteral<'a>),
     BytesLiteral(ExprBytesLiteral<'a>),
     NumberLiteral(ExprNumberLiteral<'a>),
-    BoolLiteral(ExprBoolLiteral<'a>),
+    BoolLiteral(ExprBoolLiteral),
     NoneLiteral,
     EllipsisLiteral,
     Attribute(ExprAttribute<'a>),
@@ -899,6 +1064,7 @@ impl<'a> From<&'a ast::Expr> for ComparableExpr<'a> {
                 op,
                 values,
                 range: _,
+                node_index: _,
             }) => Self::BoolOp(ExprBoolOp {
                 op: (*op).into(),
                 values: values.iter().map(Into::into).collect(),
@@ -907,6 +1073,7 @@ impl<'a> From<&'a ast::Expr> for ComparableExpr<'a> {
                 target,
                 value,
                 range: _,
+                node_index: _,
             }) => Self::NamedExpr(ExprNamed {
                 target: target.into(),
                 value: value.into(),
@@ -916,6 +1083,7 @@ impl<'a> From<&'a ast::Expr> for ComparableExpr<'a> {
                 op,
                 right,
                 range: _,
+                node_index: _,
             }) => Self::BinOp(ExprBinOp {
                 left: left.into(),
                 op: (*op).into(),
@@ -925,6 +1093,7 @@ impl<'a> From<&'a ast::Expr> for ComparableExpr<'a> {
                 op,
                 operand,
                 range: _,
+                node_index: _,
             }) => Self::UnaryOp(ExprUnaryOp {
                 op: (*op).into(),
                 operand: operand.into(),
@@ -933,6 +1102,7 @@ impl<'a> From<&'a ast::Expr> for ComparableExpr<'a> {
                 parameters,
                 body,
                 range: _,
+                node_index: _,
             }) => Self::Lambda(ExprLambda {
                 parameters: parameters.as_ref().map(Into::into),
                 body: body.into(),
@@ -942,21 +1112,31 @@ impl<'a> From<&'a ast::Expr> for ComparableExpr<'a> {
                 body,
                 orelse,
                 range: _,
+                node_index: _,
             }) => Self::IfExp(ExprIf {
                 test: test.into(),
                 body: body.into(),
                 orelse: orelse.into(),
             }),
-            ast::Expr::Dict(ast::ExprDict { items, range: _ }) => Self::Dict(ExprDict {
+            ast::Expr::Dict(ast::ExprDict {
+                items,
+                range: _,
+                node_index: _,
+            }) => Self::Dict(ExprDict {
                 items: items.iter().map(ComparableDictItem::from).collect(),
             }),
-            ast::Expr::Set(ast::ExprSet { elts, range: _ }) => Self::Set(ExprSet {
+            ast::Expr::Set(ast::ExprSet {
+                elts,
+                range: _,
+                node_index: _,
+            }) => Self::Set(ExprSet {
                 elts: elts.iter().map(Into::into).collect(),
             }),
             ast::Expr::ListComp(ast::ExprListComp {
                 elt,
                 generators,
                 range: _,
+                node_index: _,
             }) => Self::ListComp(ExprListComp {
                 elt: elt.into(),
                 generators: generators.iter().map(Into::into).collect(),
@@ -965,6 +1145,7 @@ impl<'a> From<&'a ast::Expr> for ComparableExpr<'a> {
                 elt,
                 generators,
                 range: _,
+                node_index: _,
             }) => Self::SetComp(ExprSetComp {
                 elt: elt.into(),
                 generators: generators.iter().map(Into::into).collect(),
@@ -974,6 +1155,7 @@ impl<'a> From<&'a ast::Expr> for ComparableExpr<'a> {
                 value,
                 generators,
                 range: _,
+                node_index: _,
             }) => Self::DictComp(ExprDictComp {
                 key: key.into(),
                 value: value.into(),
@@ -983,27 +1165,39 @@ impl<'a> From<&'a ast::Expr> for ComparableExpr<'a> {
                 elt,
                 generators,
                 range: _,
+                node_index: _,
                 parenthesized: _,
             }) => Self::GeneratorExp(ExprGenerator {
                 elt: elt.into(),
                 generators: generators.iter().map(Into::into).collect(),
             }),
-            ast::Expr::Await(ast::ExprAwait { value, range: _ }) => Self::Await(ExprAwait {
+            ast::Expr::Await(ast::ExprAwait {
+                value,
+                range: _,
+                node_index: _,
+            }) => Self::Await(ExprAwait {
                 value: value.into(),
             }),
-            ast::Expr::Yield(ast::ExprYield { value, range: _ }) => Self::Yield(ExprYield {
+            ast::Expr::Yield(ast::ExprYield {
+                value,
+                range: _,
+                node_index: _,
+            }) => Self::Yield(ExprYield {
                 value: value.as_ref().map(Into::into),
             }),
-            ast::Expr::YieldFrom(ast::ExprYieldFrom { value, range: _ }) => {
-                Self::YieldFrom(ExprYieldFrom {
-                    value: value.into(),
-                })
-            }
+            ast::Expr::YieldFrom(ast::ExprYieldFrom {
+                value,
+                range: _,
+                node_index: _,
+            }) => Self::YieldFrom(ExprYieldFrom {
+                value: value.into(),
+            }),
             ast::Expr::Compare(ast::ExprCompare {
                 left,
                 ops,
                 comparators,
                 range: _,
+                node_index: _,
             }) => Self::Compare(ExprCompare {
                 left: left.into(),
                 ops: ops.iter().copied().map(Into::into).collect(),
@@ -1013,33 +1207,55 @@ impl<'a> From<&'a ast::Expr> for ComparableExpr<'a> {
                 func,
                 arguments,
                 range: _,
+                node_index: _,
             }) => Self::Call(ExprCall {
                 func: func.into(),
                 arguments: arguments.into(),
             }),
-            ast::Expr::FString(ast::ExprFString { value, range: _ }) => {
-                Self::FString(ExprFString {
-                    parts: value.iter().map(Into::into).collect(),
-                })
-            }
-            ast::Expr::StringLiteral(ast::ExprStringLiteral { value, range: _ }) => {
-                Self::StringLiteral(ExprStringLiteral {
-                    parts: value.iter().map(Into::into).collect(),
-                })
-            }
-            ast::Expr::BytesLiteral(ast::ExprBytesLiteral { value, range: _ }) => {
-                Self::BytesLiteral(ExprBytesLiteral {
-                    parts: value.iter().map(Into::into).collect(),
-                })
-            }
-            ast::Expr::NumberLiteral(ast::ExprNumberLiteral { value, range: _ }) => {
-                Self::NumberLiteral(ExprNumberLiteral {
-                    value: value.into(),
-                })
-            }
-            ast::Expr::BooleanLiteral(ast::ExprBooleanLiteral { value, range: _ }) => {
-                Self::BoolLiteral(ExprBoolLiteral { value })
-            }
+            ast::Expr::FString(ast::ExprFString {
+                value,
+                range: _,
+                node_index: _,
+            }) => Self::FString(ExprFString {
+                value: value.into(),
+            }),
+            ast::Expr::TString(ast::ExprTString {
+                value,
+                range: _,
+                node_index: _,
+            }) => Self::TString(ExprTString {
+                value: value.into(),
+            }),
+            ast::Expr::StringLiteral(ast::ExprStringLiteral {
+                value,
+                range: _,
+                node_index: _,
+            }) => Self::StringLiteral(ExprStringLiteral {
+                value: ComparableStringLiteral {
+                    value: value.to_str(),
+                },
+            }),
+            ast::Expr::BytesLiteral(ast::ExprBytesLiteral {
+                value,
+                range: _,
+                node_index: _,
+            }) => Self::BytesLiteral(ExprBytesLiteral {
+                value: ComparableBytesLiteral {
+                    value: Cow::from(value),
+                },
+            }),
+            ast::Expr::NumberLiteral(ast::ExprNumberLiteral {
+                value,
+                range: _,
+                node_index: _,
+            }) => Self::NumberLiteral(ExprNumberLiteral {
+                value: value.into(),
+            }),
+            ast::Expr::BooleanLiteral(ast::ExprBooleanLiteral {
+                value,
+                range: _,
+                node_index: _,
+            }) => Self::BoolLiteral(ExprBoolLiteral { value: *value }),
             ast::Expr::NoneLiteral(_) => Self::NoneLiteral,
             ast::Expr::EllipsisLiteral(_) => Self::EllipsisLiteral,
             ast::Expr::Attribute(ast::ExprAttribute {
@@ -1047,6 +1263,7 @@ impl<'a> From<&'a ast::Expr> for ComparableExpr<'a> {
                 attr,
                 ctx: _,
                 range: _,
+                node_index: _,
             }) => Self::Attribute(ExprAttribute {
                 value: value.into(),
                 attr: attr.as_str(),
@@ -1056,6 +1273,7 @@ impl<'a> From<&'a ast::Expr> for ComparableExpr<'a> {
                 slice,
                 ctx: _,
                 range: _,
+                node_index: _,
             }) => Self::Subscript(ExprSubscript {
                 value: value.into(),
                 slice: slice.into(),
@@ -1064,6 +1282,7 @@ impl<'a> From<&'a ast::Expr> for ComparableExpr<'a> {
                 value,
                 ctx: _,
                 range: _,
+                node_index: _,
             }) => Self::Starred(ExprStarred {
                 value: value.into(),
             }),
@@ -1072,6 +1291,7 @@ impl<'a> From<&'a ast::Expr> for ComparableExpr<'a> {
                 elts,
                 ctx: _,
                 range: _,
+                node_index: _,
             }) => Self::List(ExprList {
                 elts: elts.iter().map(Into::into).collect(),
             }),
@@ -1079,6 +1299,7 @@ impl<'a> From<&'a ast::Expr> for ComparableExpr<'a> {
                 elts,
                 ctx: _,
                 range: _,
+                node_index: _,
                 parenthesized: _,
             }) => Self::Tuple(ExprTuple {
                 elts: elts.iter().map(Into::into).collect(),
@@ -1088,6 +1309,7 @@ impl<'a> From<&'a ast::Expr> for ComparableExpr<'a> {
                 upper,
                 step,
                 range: _,
+                node_index: _,
             }) => Self::Slice(ExprSlice {
                 lower: lower.as_ref().map(Into::into),
                 upper: upper.as_ref().map(Into::into),
@@ -1097,6 +1319,7 @@ impl<'a> From<&'a ast::Expr> for ComparableExpr<'a> {
                 kind,
                 value,
                 range: _,
+                node_index: _,
             }) => Self::IpyEscapeCommand(ExprIpyEscapeCommand { kind: *kind, value }),
         }
     }
@@ -1181,6 +1404,7 @@ impl<'a> From<&'a ast::TypeParam> for ComparableTypeParam<'a> {
                 bound,
                 default,
                 range: _,
+                node_index: _,
             }) => Self::TypeVar(TypeParamTypeVar {
                 name: name.as_str(),
                 bound: bound.as_ref().map(Into::into),
@@ -1190,6 +1414,7 @@ impl<'a> From<&'a ast::TypeParam> for ComparableTypeParam<'a> {
                 name,
                 default,
                 range: _,
+                node_index: _,
             }) => Self::TypeVarTuple(TypeParamTypeVarTuple {
                 name: name.as_str(),
                 default: default.as_ref().map(Into::into),
@@ -1198,6 +1423,7 @@ impl<'a> From<&'a ast::TypeParam> for ComparableTypeParam<'a> {
                 name,
                 default,
                 range: _,
+                node_index: _,
             }) => Self::ParamSpec(TypeParamParamSpec {
                 name: name.as_str(),
                 default: default.as_ref().map(Into::into),
@@ -1377,6 +1603,7 @@ impl<'a> From<&'a ast::Stmt> for ComparableStmt<'a> {
                 returns,
                 type_params,
                 range: _,
+                node_index: _,
             }) => Self::FunctionDef(StmtFunctionDef {
                 is_async: *is_async,
                 name: name.as_str(),
@@ -1393,6 +1620,7 @@ impl<'a> From<&'a ast::Stmt> for ComparableStmt<'a> {
                 decorator_list,
                 type_params,
                 range: _,
+                node_index: _,
             }) => Self::ClassDef(StmtClassDef {
                 name: name.as_str(),
                 arguments: arguments.as_ref().map(Into::into).unwrap_or_default(),
@@ -1400,14 +1628,23 @@ impl<'a> From<&'a ast::Stmt> for ComparableStmt<'a> {
                 decorator_list: decorator_list.iter().map(Into::into).collect(),
                 type_params: type_params.as_ref().map(Into::into),
             }),
-            ast::Stmt::Return(ast::StmtReturn { value, range: _ }) => Self::Return(StmtReturn {
+            ast::Stmt::Return(ast::StmtReturn {
+                value,
+                range: _,
+                node_index: _,
+            }) => Self::Return(StmtReturn {
                 value: value.as_ref().map(Into::into),
             }),
-            ast::Stmt::Delete(ast::StmtDelete { targets, range: _ }) => Self::Delete(StmtDelete {
+            ast::Stmt::Delete(ast::StmtDelete {
+                targets,
+                range: _,
+                node_index: _,
+            }) => Self::Delete(StmtDelete {
                 targets: targets.iter().map(Into::into).collect(),
             }),
             ast::Stmt::TypeAlias(ast::StmtTypeAlias {
                 range: _,
+                node_index: _,
                 name,
                 type_params,
                 value,
@@ -1420,6 +1657,7 @@ impl<'a> From<&'a ast::Stmt> for ComparableStmt<'a> {
                 targets,
                 value,
                 range: _,
+                node_index: _,
             }) => Self::Assign(StmtAssign {
                 targets: targets.iter().map(Into::into).collect(),
                 value: value.into(),
@@ -1429,6 +1667,7 @@ impl<'a> From<&'a ast::Stmt> for ComparableStmt<'a> {
                 op,
                 value,
                 range: _,
+                node_index: _,
             }) => Self::AugAssign(StmtAugAssign {
                 target: target.into(),
                 op: (*op).into(),
@@ -1440,6 +1679,7 @@ impl<'a> From<&'a ast::Stmt> for ComparableStmt<'a> {
                 value,
                 simple,
                 range: _,
+                node_index: _,
             }) => Self::AnnAssign(StmtAnnAssign {
                 target: target.into(),
                 annotation: annotation.into(),
@@ -1453,6 +1693,7 @@ impl<'a> From<&'a ast::Stmt> for ComparableStmt<'a> {
                 body,
                 orelse,
                 range: _,
+                node_index: _,
             }) => Self::For(StmtFor {
                 is_async: *is_async,
                 target: target.into(),
@@ -1465,6 +1706,7 @@ impl<'a> From<&'a ast::Stmt> for ComparableStmt<'a> {
                 body,
                 orelse,
                 range: _,
+                node_index: _,
             }) => Self::While(StmtWhile {
                 test: test.into(),
                 body: body.iter().map(Into::into).collect(),
@@ -1475,6 +1717,7 @@ impl<'a> From<&'a ast::Stmt> for ComparableStmt<'a> {
                 body,
                 elif_else_clauses,
                 range: _,
+                node_index: _,
             }) => Self::If(StmtIf {
                 test: test.into(),
                 body: body.iter().map(Into::into).collect(),
@@ -1485,6 +1728,7 @@ impl<'a> From<&'a ast::Stmt> for ComparableStmt<'a> {
                 items,
                 body,
                 range: _,
+                node_index: _,
             }) => Self::With(StmtWith {
                 is_async: *is_async,
                 items: items.iter().map(Into::into).collect(),
@@ -1494,6 +1738,7 @@ impl<'a> From<&'a ast::Stmt> for ComparableStmt<'a> {
                 subject,
                 cases,
                 range: _,
+                node_index: _,
             }) => Self::Match(StmtMatch {
                 subject: subject.into(),
                 cases: cases.iter().map(Into::into).collect(),
@@ -1502,6 +1747,7 @@ impl<'a> From<&'a ast::Stmt> for ComparableStmt<'a> {
                 exc,
                 cause,
                 range: _,
+                node_index: _,
             }) => Self::Raise(StmtRaise {
                 exc: exc.as_ref().map(Into::into),
                 cause: cause.as_ref().map(Into::into),
@@ -1513,6 +1759,7 @@ impl<'a> From<&'a ast::Stmt> for ComparableStmt<'a> {
                 finalbody,
                 is_star,
                 range: _,
+                node_index: _,
             }) => Self::Try(StmtTry {
                 body: body.iter().map(Into::into).collect(),
                 handlers: handlers.iter().map(Into::into).collect(),
@@ -1524,11 +1771,16 @@ impl<'a> From<&'a ast::Stmt> for ComparableStmt<'a> {
                 test,
                 msg,
                 range: _,
+                node_index: _,
             }) => Self::Assert(StmtAssert {
                 test: test.into(),
                 msg: msg.as_ref().map(Into::into),
             }),
-            ast::Stmt::Import(ast::StmtImport { names, range: _ }) => Self::Import(StmtImport {
+            ast::Stmt::Import(ast::StmtImport {
+                names,
+                range: _,
+                node_index: _,
+            }) => Self::Import(StmtImport {
                 names: names.iter().map(Into::into).collect(),
             }),
             ast::Stmt::ImportFrom(ast::StmtImportFrom {
@@ -1536,25 +1788,37 @@ impl<'a> From<&'a ast::Stmt> for ComparableStmt<'a> {
                 names,
                 level,
                 range: _,
+                node_index: _,
             }) => Self::ImportFrom(StmtImportFrom {
                 module: module.as_deref(),
                 names: names.iter().map(Into::into).collect(),
                 level: *level,
             }),
-            ast::Stmt::Global(ast::StmtGlobal { names, range: _ }) => Self::Global(StmtGlobal {
+            ast::Stmt::Global(ast::StmtGlobal {
+                names,
+                range: _,
+                node_index: _,
+            }) => Self::Global(StmtGlobal {
+                names: names.iter().map(ast::Identifier::as_str).collect(),
+            }),
+            ast::Stmt::Nonlocal(ast::StmtNonlocal {
+                names,
+                range: _,
+                node_index: _,
+            }) => Self::Nonlocal(StmtNonlocal {
                 names: names.iter().map(ast::Identifier::as_str).collect(),
             }),
-            ast::Stmt::Nonlocal(ast::StmtNonlocal { names, range: _ }) => {
-                Self::Nonlocal(StmtNonlocal {
-                    names: names.iter().map(ast::Identifier::as_str).collect(),
-                })
-            }
             ast::Stmt::IpyEscapeCommand(ast::StmtIpyEscapeCommand {
                 kind,
                 value,
                 range: _,
+                node_index: _,
             }) => Self::IpyEscapeCommand(StmtIpyEscapeCommand { kind: *kind, value }),
-            ast::Stmt::Expr(ast::StmtExpr { value, range: _ }) => Self::Expr(StmtExpr {
+            ast::Stmt::Expr(ast::StmtExpr {
+                value,
+                range: _,
+                node_index: _,
+            }) => Self::Expr(StmtExpr {
                 value: value.into(),
             }),
             ast::Stmt::Pass(_) => Self::Pass,
@@ -1604,3 +1868,76 @@ impl<'a> From<&'a ast::ModExpression> for ComparableModExpression<'a> {
         }
     }
 }
+
+/// Wrapper around [`Expr`] that implements [`Hash`] and [`PartialEq`] according to Python
+/// semantics:
+///
+/// > Values that compare equal (such as 1, 1.0, and True) can be used interchangeably to index the
+/// > same dictionary entry.
+///
+/// For example, considers `True`, `1`, and `1.0` to be equal, as they hash to the same value
+/// in Python, along with `False`, `0`, and `0.0`.
+///
+/// See: <https://docs.python.org/3/library/stdtypes.html#mapping-types-dict>
+#[derive(Debug)]
+pub struct HashableExpr<'a>(ComparableExpr<'a>);
+
+impl Hash for HashableExpr<'_> {
+    fn hash<H: std::hash::Hasher>(&self, state: &mut H) {
+        self.0.hash(state);
+    }
+}
+
+impl PartialEq<Self> for HashableExpr<'_> {
+    fn eq(&self, other: &Self) -> bool {
+        self.0 == other.0
+    }
+}
+
+impl Eq for HashableExpr<'_> {}
+
+impl<'a> From<&'a Expr> for HashableExpr<'a> {
+    fn from(expr: &'a Expr) -> Self {
+        /// Returns a version of the given expression that can be hashed and compared according to
+        /// Python  semantics.
+        fn as_hashable(expr: &Expr) -> ComparableExpr {
+            match expr {
+                Expr::Named(named) => ComparableExpr::NamedExpr(ExprNamed {
+                    target: Box::new(ComparableExpr::from(&named.target)),
+                    value: Box::new(as_hashable(&named.value)),
+                }),
+                Expr::NumberLiteral(number) => as_bool(number)
+                    .map(|value| ComparableExpr::BoolLiteral(ExprBoolLiteral { value }))
+                    .unwrap_or_else(|| ComparableExpr::from(expr)),
+                Expr::Tuple(tuple) => ComparableExpr::Tuple(ExprTuple {
+                    elts: tuple.iter().map(as_hashable).collect(),
+                }),
+                _ => ComparableExpr::from(expr),
+            }
+        }
+
+        /// Returns the `bool` value of the given expression, if it has an equivalent hash to
+        /// `True` or `False`.
+        fn as_bool(number: &crate::ExprNumberLiteral) -> Option<bool> {
+            match &number.value {
+                Number::Int(int) => match int.as_u8() {
+                    Some(0) => Some(false),
+                    Some(1) => Some(true),
+                    _ => None,
+                },
+                Number::Float(float) => match float {
+                    0.0 => Some(false),
+                    1.0 => Some(true),
+                    _ => None,
+                },
+                Number::Complex { real, imag } => match (real, imag) {
+                    (0.0, 0.0) => Some(false),
+                    (1.0, 0.0) => Some(true),
+                    _ => None,
+                },
+            }
+        }
+
+        Self(as_hashable(expr))
+    }
+}
