@@ -850,58 +850,58 @@ mod tests {
     }
 
     #[test]
-    fn test_parse_t_string_concat_1() {
+    fn test_parse_t_string_concat_1_error() {
         let source = "'Hello ' t'world'";
-        let suite = parse_suite(source).unwrap();
+        let suite = parse_suite(source).unwrap_err();
         insta::assert_debug_snapshot!(suite);
     }
 
     #[test]
-    fn test_parse_t_string_concat_2() {
+    fn test_parse_t_string_concat_2_error() {
         let source = "'Hello ' t'world'";
-        let suite = parse_suite(source).unwrap();
+        let suite = parse_suite(source).unwrap_err();
         insta::assert_debug_snapshot!(suite);
     }
 
     #[test]
-    fn test_parse_t_string_concat_3() {
+    fn test_parse_t_string_concat_3_error() {
         let source = "'Hello ' t'world{\"!\"}'";
-        let suite = parse_suite(source).unwrap();
+        let suite = parse_suite(source).unwrap_err();
         insta::assert_debug_snapshot!(suite);
     }
 
     #[test]
-    fn test_parse_t_string_concat_4() {
+    fn test_parse_t_string_concat_4_error() {
         let source = "'Hello ' t'world{\"!\"}' 'again!'";
-        let suite = parse_suite(source).unwrap();
+        let suite = parse_suite(source).unwrap_err();
         insta::assert_debug_snapshot!(suite);
     }
 
     #[test]
-    fn test_parse_u_t_string_concat_1() {
+    fn test_parse_u_t_string_concat_1_error() {
         let source = "u'Hello ' t'world'";
-        let suite = parse_suite(source).unwrap();
+        let suite = parse_suite(source).unwrap_err();
         insta::assert_debug_snapshot!(suite);
     }
 
     #[test]
-    fn test_parse_u_t_string_concat_2() {
+    fn test_parse_u_t_string_concat_2_error() {
         let source = "u'Hello ' t'world' '!'";
-        let suite = parse_suite(source).unwrap();
+        let suite = parse_suite(source).unwrap_err();
         insta::assert_debug_snapshot!(suite);
     }
 
     #[test]
-    fn test_parse_f_t_string_concat_1() {
+    fn test_parse_f_t_string_concat_1_error() {
         let source = "f'Hello ' t'world'";
-        let suite = parse_suite(source).unwrap();
+        let suite = parse_suite(source).unwrap_err();
         insta::assert_debug_snapshot!(suite);
     }
 
     #[test]
-    fn test_parse_f_t_string_concat_2() {
+    fn test_parse_f_t_string_concat_2_error() {
         let source = "f'Hello ' t'world' '!'";
-        let suite = parse_suite(source).unwrap();
+        let suite = parse_suite(source).unwrap_err();
         insta::assert_debug_snapshot!(suite);
     }
 
