@@ -1,9 +1,8 @@
-use crate::server::api::diagnostics::publish_diagnostics_for_document;
-use crate::server::api::LSPResult;
-use crate::server::client::{Notifier, Requester};
-use crate::server::Result;
-use crate::session::Session;
 use crate::TextDocument;
+use crate::server::Result;
+use crate::server::api::LSPResult;
+use crate::server::api::diagnostics::publish_diagnostics_for_document;
+use crate::session::{Client, Session};
 use lsp_types as types;
 use lsp_types::notification as notif;
 
@@ -16,20 +15,22 @@ impl super::NotificationHandler for DidOpen {
 impl super::SyncNotificationHandler for DidOpen {
     fn run(
         session: &mut Session,
-        notifier: Notifier,
-        _requester: &mut Requester,
+        client: &Client,
         types::DidOpenTextDocumentParams {
             text_document:
                 types::TextDocumentItem {
-                    uri, text, version, ..
+                    uri,
+                    text,
+                    version,
+                    language_id,
                 },
         }: types::DidOpenTextDocumentParams,
     ) -> Result<()> {
-        let document = TextDocument::new(text, version);
+        let document = TextDocument::new(text, version).with_language_id(&language_id);
 
         session.open_text_document(uri.clone(), document);
 
-        // Publish diagnostics if the client doesnt support pull diagnostics
+        // Publish diagnostics if the client doesn't support pull diagnostics
         if !session.resolved_client_capabilities().pull_diagnostics {
             let snapshot = session
                 .take_snapshot(uri.clone())
@@ -37,7 +38,7 @@ impl super::SyncNotificationHandler for DidOpen {
                     anyhow::anyhow!("Unable to take snapshot for document with URL {uri}")
                 })
                 .with_failure_code(lsp_server::ErrorCode::InternalError)?;
-            publish_diagnostics_for_document(&snapshot, &notifier)?;
+            publish_diagnostics_for_document(&snapshot, client)?;
         }
 
         Ok(())
