@@ -1,24 +1,29 @@
 use std::borrow::Cow;
+use std::ops::{Deref, DerefMut};
 use std::path::PathBuf;
 use std::{collections::BTreeMap, path::Path, sync::Arc};
 
 use anyhow::anyhow;
-use lsp_types::Url;
-use rustc_hash::FxHashMap;
+use lsp_types::{FileEvent, Url};
+use rustc_hash::{FxHashMap, FxHashSet};
+use thiserror::Error;
 
 pub(crate) use ruff_settings::RuffSettings;
 
+use crate::edit::LanguageId;
+use crate::session::Client;
+use crate::session::options::Combine;
+use crate::session::settings::GlobalClientSettings;
+use crate::workspace::{Workspace, Workspaces};
 use crate::{
-    edit::{DocumentKey, DocumentVersion, NotebookDocument},
     PositionEncoding, TextDocument,
+    edit::{DocumentKey, DocumentVersion, NotebookDocument},
 };
 
-use super::{settings::ResolvedClientSettings, ClientSettings};
+use super::settings::ClientSettings;
 
 mod ruff_settings;
 
-type SettingsIndex = BTreeMap<PathBuf, WorkspaceSettings>;
-
 /// Stores and tracks all open documents in a session, along with their associated settings.
 #[derive(Default)]
 pub(crate) struct Index {
@@ -29,12 +34,12 @@ pub(crate) struct Index {
     notebook_cells: FxHashMap<Url, Url>,
 
     /// Maps a workspace folder root to its settings.
-    settings: SettingsIndex,
+    settings: WorkspaceSettingsIndex,
 }
 
 /// Settings associated with a workspace.
 struct WorkspaceSettings {
-    client_settings: ResolvedClientSettings,
+    client_settings: Arc<ClientSettings>,
     ruff_settings: ruff_settings::RuffSettingsIndex,
 }
 
@@ -67,23 +72,19 @@ pub enum DocumentQuery {
 
 impl Index {
     pub(super) fn new(
-        workspace_folders: Vec<(Url, ClientSettings)>,
-        global_settings: &ClientSettings,
+        workspaces: &Workspaces,
+        global: &GlobalClientSettings,
+        client: &Client,
     ) -> crate::Result<Self> {
-        let mut settings_index = BTreeMap::new();
-        for (url, workspace_settings) in workspace_folders {
-            Self::register_workspace_settings(
-                &mut settings_index,
-                &url,
-                Some(workspace_settings),
-                global_settings,
-            )?;
+        let mut settings = WorkspaceSettingsIndex::default();
+        for workspace in &**workspaces {
+            settings.register_workspace(workspace, global, client)?;
         }
 
         Ok(Self {
             documents: FxHashMap::default(),
             notebook_cells: FxHashMap::default(),
-            settings: settings_index,
+            settings,
         })
     }
 
@@ -128,7 +129,7 @@ impl Index {
             DocumentKey::NotebookCell(url)
         } else if Path::new(url.path())
             .extension()
-            .map_or(false, |ext| ext.eq_ignore_ascii_case("ipynb"))
+            .is_some_and(|ext| ext.eq_ignore_ascii_case("ipynb"))
         {
             DocumentKey::Notebook(url)
         } else {
@@ -172,64 +173,19 @@ impl Index {
 
     pub(super) fn open_workspace_folder(
         &mut self,
-        url: &Url,
-        global_settings: &ClientSettings,
+        url: Url,
+        global: &GlobalClientSettings,
+        client: &Client,
     ) -> crate::Result<()> {
         // TODO(jane): Find a way for workspace client settings to be added or changed dynamically.
-        Self::register_workspace_settings(&mut self.settings, url, None, global_settings)
-    }
-
-    pub(super) fn num_documents(&self) -> usize {
-        self.documents.len()
-    }
-
-    pub(super) fn num_workspaces(&self) -> usize {
-        self.settings.len()
-    }
-
-    pub(super) fn list_config_files(&self) -> Vec<&Path> {
         self.settings
-            .values()
-            .flat_map(|WorkspaceSettings { ruff_settings, .. }| ruff_settings.list_files())
-            .collect()
-    }
-
-    fn register_workspace_settings(
-        settings_index: &mut SettingsIndex,
-        workspace_url: &Url,
-        workspace_settings: Option<ClientSettings>,
-        global_settings: &ClientSettings,
-    ) -> crate::Result<()> {
-        let client_settings = if let Some(workspace_settings) = workspace_settings {
-            ResolvedClientSettings::with_workspace(&workspace_settings, global_settings)
-        } else {
-            ResolvedClientSettings::global(global_settings)
-        };
-
-        let workspace_path = workspace_url
-            .to_file_path()
-            .map_err(|()| anyhow!("workspace URL was not a file path!"))?;
-
-        let workspace_settings_index = ruff_settings::RuffSettingsIndex::new(
-            &workspace_path,
-            client_settings.editor_settings(),
-        );
-
-        settings_index.insert(
-            workspace_path,
-            WorkspaceSettings {
-                client_settings,
-                ruff_settings: workspace_settings_index,
-            },
-        );
-
-        Ok(())
+            .register_workspace(&Workspace::new(url), global, client)
     }
 
     pub(super) fn close_workspace_folder(&mut self, workspace_url: &Url) -> crate::Result<()> {
-        let workspace_path = workspace_url
-            .to_file_path()
-            .map_err(|()| anyhow!("workspace URL was not a file path!"))?;
+        let workspace_path = workspace_url.to_file_path().map_err(|()| {
+            anyhow!("Failed to convert workspace URL to file path: {workspace_url}")
+        })?;
 
         self.settings.remove(&workspace_path).ok_or_else(|| {
             anyhow!(
@@ -250,7 +206,7 @@ impl Index {
     pub(super) fn make_document_ref(
         &self,
         key: DocumentKey,
-        global_settings: &ClientSettings,
+        global: &GlobalClientSettings,
     ) -> Option<DocumentQuery> {
         let url = self.url_for_key(&key)?.clone();
 
@@ -279,13 +235,12 @@ impl Index {
                     "No settings available for {} - falling back to default settings",
                     url
                 );
-                let resolved_global = ResolvedClientSettings::global(global_settings);
                 // The path here is only for completeness, it's okay to use a non-existing path
                 // in case this is an unsaved (untitled) document.
                 let path = Path::new(url.path());
                 let root = path.parent().unwrap_or(path);
                 Arc::new(RuffSettings::fallback(
-                    resolved_global.editor_settings(),
+                    global.to_settings().editor_settings(),
                     root,
                 ))
             });
@@ -298,29 +253,49 @@ impl Index {
         Some(controller.make_ref(cell_url, url, document_settings))
     }
 
-    /// Reloads relevant existing settings files based on a changed settings file path.
-    /// This does not currently register new settings files.
-    pub(super) fn reload_settings(&mut self, changed_url: &Url) {
-        let Ok(changed_path) = changed_url.to_file_path() else {
-            // Files that don't map to a path can't be a workspace configuration file.
-            return;
-        };
+    /// Reload the settings for all the workspace folders that contain the changed files.
+    ///
+    /// This method avoids re-indexing the same workspace multiple times if multiple files
+    /// belonging to the same workspace have been changed.
+    ///
+    /// The file events are expected to only contain paths that map to configuration files. This is
+    /// registered in [`try_register_capabilities`] method.
+    ///
+    /// [`try_register_capabilities`]: crate::server::Server::try_register_capabilities
+    pub(super) fn reload_settings(&mut self, changes: &[FileEvent], client: &Client) {
+        let mut indexed = FxHashSet::default();
 
-        let Some(enclosing_folder) = changed_path.parent() else {
-            return;
-        };
+        for change in changes {
+            let Ok(changed_path) = change.uri.to_file_path() else {
+                // Files that don't map to a path can't be a workspace configuration file.
+                return;
+            };
 
-        // TODO: I think this does not correctly reload settings when using `extend` and the extended
-        //  setting isn't in a parent folder.
-        for (root, settings) in self.settings.range_mut(enclosing_folder.to_path_buf()..) {
-            if !root.starts_with(enclosing_folder) {
-                break;
+            let Some(enclosing_folder) = changed_path.parent() else {
+                return;
+            };
+
+            for (root, settings) in self
+                .settings
+                .range_mut(..=enclosing_folder.to_path_buf())
+                .rev()
+            {
+                if !enclosing_folder.starts_with(root) {
+                    break;
+                }
+
+                if indexed.contains(root) {
+                    continue;
+                }
+                indexed.insert(root.clone());
+
+                settings.ruff_settings = ruff_settings::RuffSettingsIndex::new(
+                    client,
+                    root,
+                    settings.client_settings.editor_settings(),
+                    false,
+                );
             }
-
-            settings.ruff_settings = ruff_settings::RuffSettingsIndex::new(
-                root,
-                settings.client_settings.editor_settings(),
-            );
         }
     }
 
@@ -360,21 +335,12 @@ impl Index {
         Ok(())
     }
 
-    pub(super) fn client_settings(
-        &self,
-        key: &DocumentKey,
-        global_settings: &ClientSettings,
-    ) -> ResolvedClientSettings {
-        let Some(url) = self.url_for_key(key) else {
-            return ResolvedClientSettings::global(global_settings);
-        };
-        let Some(WorkspaceSettings {
+    pub(super) fn client_settings(&self, key: &DocumentKey) -> Option<Arc<ClientSettings>> {
+        let url = self.url_for_key(key)?;
+        let WorkspaceSettings {
             client_settings, ..
-        }) = self.settings_for_url(url)
-        else {
-            return ResolvedClientSettings::global(global_settings);
-        };
-        client_settings.clone()
+        } = self.settings_for_url(url)?;
+        Some(client_settings.clone())
     }
 
     fn document_controller_for_key(
@@ -419,6 +385,103 @@ impl Index {
             .next_back()
             .map(|(_, settings)| settings)
     }
+
+    /// Returns an iterator over the workspace root folders contained in this index.
+    pub(super) fn workspace_root_folders(&self) -> impl Iterator<Item = &Path> {
+        self.settings.keys().map(PathBuf::as_path)
+    }
+
+    /// Returns the number of open documents.
+    pub(super) fn open_documents_len(&self) -> usize {
+        self.documents.len()
+    }
+
+    /// Returns an iterator over the paths to the configuration files in the index.
+    pub(super) fn config_file_paths(&self) -> impl Iterator<Item = &Path> {
+        self.settings
+            .values()
+            .flat_map(|WorkspaceSettings { ruff_settings, .. }| ruff_settings.config_file_paths())
+    }
+}
+
+/// Maps a workspace folder root to its settings.
+#[derive(Default)]
+struct WorkspaceSettingsIndex {
+    index: BTreeMap<PathBuf, WorkspaceSettings>,
+}
+
+impl WorkspaceSettingsIndex {
+    /// Register a workspace folder with the given settings.
+    ///
+    /// If the `workspace_settings` is [`Some`], it is preferred over the global settings for the
+    /// workspace. Otherwise, the global settings are used exclusively.
+    fn register_workspace(
+        &mut self,
+        workspace: &Workspace,
+        global: &GlobalClientSettings,
+        client: &Client,
+    ) -> crate::Result<()> {
+        let workspace_url = workspace.url();
+        if workspace_url.scheme() != "file" {
+            tracing::info!("Ignoring non-file workspace URL: {workspace_url}");
+            client.show_warning_message(format_args!(
+                "Ruff does not support non-file workspaces; Ignoring {workspace_url}"
+            ));
+            return Ok(());
+        }
+        let workspace_path = workspace_url.to_file_path().map_err(|()| {
+            anyhow!("Failed to convert workspace URL to file path: {workspace_url}")
+        })?;
+
+        let client_settings = if let Some(workspace_options) = workspace.options() {
+            let options = workspace_options.clone().combine(global.options().clone());
+            let settings = match options.into_settings() {
+                Ok(settings) => settings,
+                Err(settings) => {
+                    client.show_error_message(format_args!(
+                        "The settings for the workspace {workspace_path} are invalid. Refer to the logs for more information.",
+                        workspace_path = workspace_path.display()
+                    ));
+                    settings
+                }
+            };
+            Arc::new(settings)
+        } else {
+            global.to_settings_arc()
+        };
+
+        let workspace_settings_index = ruff_settings::RuffSettingsIndex::new(
+            client,
+            &workspace_path,
+            client_settings.editor_settings(),
+            workspace.is_default(),
+        );
+
+        tracing::info!("Registering workspace: {}", workspace_path.display());
+        self.insert(
+            workspace_path,
+            WorkspaceSettings {
+                client_settings,
+                ruff_settings: workspace_settings_index,
+            },
+        );
+
+        Ok(())
+    }
+}
+
+impl Deref for WorkspaceSettingsIndex {
+    type Target = BTreeMap<PathBuf, WorkspaceSettings>;
+
+    fn deref(&self) -> &Self::Target {
+        &self.index
+    }
+}
+
+impl DerefMut for WorkspaceSettingsIndex {
+    fn deref_mut(&mut self) -> &mut Self::Target {
+        &mut self.index
+    }
 }
 
 impl DocumentController {
@@ -465,7 +528,6 @@ impl DocumentController {
         }
     }
 
-    #[allow(dead_code)]
     pub(crate) fn as_text(&self) -> Option<&TextDocument> {
         match self {
             Self::Text(document) => Some(document),
@@ -508,7 +570,7 @@ impl DocumentQuery {
                 ruff_linter::source_kind::SourceKind::Python(document.contents().to_string())
             }
             Self::Notebook { notebook, .. } => {
-                ruff_linter::source_kind::SourceKind::IpyNotebook(notebook.make_ruff_notebook())
+                ruff_linter::source_kind::SourceKind::ipy_notebook(notebook.make_ruff_notebook())
             }
         }
     }
@@ -565,16 +627,40 @@ impl DocumentQuery {
 
     /// Attempt to access the single inner text document selected by the query.
     /// If this query is selecting an entire notebook document, this will return `None`.
-    pub(crate) fn as_single_document(&self) -> Option<&TextDocument> {
+    pub(crate) fn as_single_document(&self) -> Result<&TextDocument, SingleDocumentError> {
         match self {
-            Self::Text { document, .. } => Some(document),
+            Self::Text { document, .. } => Ok(document),
             Self::Notebook {
                 notebook,
+                file_url,
                 cell_url: cell_uri,
                 ..
-            } => cell_uri
-                .as_ref()
-                .and_then(|cell_uri| notebook.cell_document_by_uri(cell_uri)),
+            } => {
+                if let Some(cell_uri) = cell_uri {
+                    let cell = notebook
+                        .cell_document_by_uri(cell_uri)
+                        .ok_or_else(|| SingleDocumentError::CellDoesNotExist(cell_uri.clone()))?;
+                    Ok(cell)
+                } else {
+                    Err(SingleDocumentError::Notebook(file_url.clone()))
+                }
+            }
+        }
+    }
+
+    pub(crate) fn text_document_language_id(&self) -> Option<LanguageId> {
+        if let DocumentQuery::Text { document, .. } = self {
+            document.language_id()
+        } else {
+            None
         }
     }
 }
+
+#[derive(Debug, Error)]
+pub(crate) enum SingleDocumentError {
+    #[error("Expected a single text document, but found a notebook document: {0}")]
+    Notebook(Url),
+    #[error("Cell with URL {0} does not exist in the internal notebook document")]
+    CellDoesNotExist(Url),
+}
