@@ -20,7 +20,7 @@ use crate::types::{SpecialFormType, Type, protocol_class::ProtocolClassLiteral};
 use crate::util::diagnostics::format_enumeration;
 use crate::{Db, FxIndexMap, Module, ModuleName, Program, declare_lint};
 use itertools::Itertools;
-use ruff_db::diagnostic::{Annotation, Diagnostic, Severity, SubDiagnostic};
+use ruff_db::diagnostic::{Annotation, Diagnostic, SubDiagnostic, SubDiagnosticSeverity};
 use ruff_python_ast::{self as ast, AnyNodeRef};
 use ruff_text_size::{Ranged, TextRange};
 use rustc_hash::FxHashSet;
@@ -34,6 +34,7 @@ pub(crate) fn register_lints(registry: &mut LintRegistryBuilder) {
     registry.register_lint(&CONFLICTING_DECLARATIONS);
     registry.register_lint(&CONFLICTING_METACLASS);
     registry.register_lint(&CYCLIC_CLASS_DEFINITION);
+    registry.register_lint(&DEPRECATED);
     registry.register_lint(&DIVISION_BY_ZERO);
     registry.register_lint(&DUPLICATE_BASE);
     registry.register_lint(&DUPLICATE_KW_ONLY);
@@ -261,6 +262,27 @@ declare_lint! {
     }
 }
 
+declare_lint! {
+    /// ## What it does
+    /// Checks for uses of deprecated items
+    ///
+    /// ## Why is this bad?
+    /// Deprecated items should no longer be used.
+    ///
+    /// ## Examples
+    /// ```python
+    /// @warnings.deprecated("use new_func instead")
+    /// def old_func(): ...
+    ///
+    /// old_func()  # emits [deprecated] diagnostic
+    /// ```
+    pub(crate) static DEPRECATED = {
+        summary: "detects uses of deprecated items",
+        status: LintStatus::preview("1.0.0"),
+        default_level: Level::Warn,
+    }
+}
+
 declare_lint! {
     /// ## What it does
     /// Checks for class definitions with duplicate bases.
@@ -1649,18 +1671,26 @@ impl TypeCheckDiagnostics {
         self.diagnostics.shrink_to_fit();
     }
 
-    pub(crate) fn into_vec(self) -> Vec<Diagnostic> {
+    pub(crate) fn into_diagnostics(self) -> Vec<Diagnostic> {
         self.diagnostics
     }
 
+    pub(crate) fn is_empty(&self) -> bool {
+        self.diagnostics.is_empty() && self.used_suppressions.is_empty()
+    }
+
     pub fn iter(&self) -> std::slice::Iter<'_, Diagnostic> {
-        self.diagnostics.iter()
+        self.diagnostics().iter()
+    }
+
+    fn diagnostics(&self) -> &[Diagnostic] {
+        self.diagnostics.as_slice()
     }
 }
 
 impl std::fmt::Debug for TypeCheckDiagnostics {
     fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
-        self.diagnostics.fmt(f)
+        self.diagnostics().fmt(f)
     }
 }
 
@@ -1669,7 +1699,7 @@ impl IntoIterator for TypeCheckDiagnostics {
     type IntoIter = std::vec::IntoIter<Diagnostic>;
 
     fn into_iter(self) -> Self::IntoIter {
-        self.diagnostics.into_iter()
+        self.into_diagnostics().into_iter()
     }
 }
 
@@ -1677,8 +1707,9 @@ impl<'a> IntoIterator for &'a TypeCheckDiagnostics {
     type Item = &'a Diagnostic;
     type IntoIter = std::slice::Iter<'a, Diagnostic>;
 
+    #[inline]
     fn into_iter(self) -> Self::IntoIter {
-        self.diagnostics.iter()
+        self.iter()
     }
 }
 
@@ -1898,7 +1929,7 @@ pub(super) fn report_implicit_return_type(
         ));
 
         let mut sub_diagnostic = SubDiagnostic::new(
-            Severity::Info,
+            SubDiagnosticSeverity::Info,
             "Only classes that directly inherit from `typing.Protocol` \
             or `typing_extensions.Protocol` are considered protocol classes",
         );
@@ -2006,7 +2037,7 @@ pub(crate) fn report_instance_layout_conflict(
     ));
 
     let mut subdiagnostic = SubDiagnostic::new(
-        Severity::Info,
+        SubDiagnosticSeverity::Info,
         "Two classes cannot coexist in a class's MRO if their instances \
         have incompatible memory layouts",
     );
@@ -2199,7 +2230,7 @@ pub(crate) fn report_bad_argument_to_get_protocol_members(
     diagnostic.info("Only protocol classes can be passed to `get_protocol_members`");
 
     let mut class_def_diagnostic = SubDiagnostic::new(
-        Severity::Info,
+        SubDiagnosticSeverity::Info,
         format_args!(
             "`{}` is declared here, but it is not a protocol class:",
             class.name(db)
@@ -2261,7 +2292,7 @@ pub(crate) fn report_runtime_check_against_non_runtime_checkable_protocol(
     diagnostic.set_primary_message("This call will raise `TypeError` at runtime");
 
     let mut class_def_diagnostic = SubDiagnostic::new(
-        Severity::Info,
+        SubDiagnosticSeverity::Info,
         format_args!(
             "`{class_name}` is declared as a protocol class, \
                 but it is not declared as runtime-checkable"
@@ -2295,7 +2326,7 @@ pub(crate) fn report_attempted_protocol_instantiation(
     diagnostic.set_primary_message("This call will raise `TypeError` at runtime");
 
     let mut class_def_diagnostic = SubDiagnostic::new(
-        Severity::Info,
+        SubDiagnosticSeverity::Info,
         format_args!("Protocol classes cannot be instantiated"),
     );
     class_def_diagnostic.annotate(
@@ -2329,7 +2360,7 @@ pub(crate) fn report_duplicate_bases(
         builder.into_diagnostic(format_args!("Duplicate base class `{duplicate_name}`",));
 
     let mut sub_diagnostic = SubDiagnostic::new(
-        Severity::Info,
+        SubDiagnosticSeverity::Info,
         format_args!(
             "The definition of class `{}` will raise `TypeError` at runtime",
             class.name(db)
@@ -2491,9 +2522,9 @@ pub(super) fn hint_if_stdlib_submodule_exists_on_other_versions(
     db: &dyn Db,
     mut diagnostic: LintDiagnosticGuard,
     full_submodule_name: &ModuleName,
-    parent_module: &Module,
+    parent_module: Module,
 ) {
-    let Some(search_path) = parent_module.search_path() else {
+    let Some(search_path) = parent_module.search_path(db) else {
         return;
     };
 
@@ -2516,7 +2547,7 @@ pub(super) fn hint_if_stdlib_submodule_exists_on_other_versions(
     diagnostic.info(format_args!(
         "The stdlib module `{module_name}` only has a `{name}` \
             submodule on Python {version_range}",
-        module_name = parent_module.name(),
+        module_name = parent_module.name(db),
         name = full_submodule_name
             .components()
             .next_back()
