@@ -82,9 +82,8 @@ pub(crate) fn enum_metadata<'db>(
     // TODO: handle `StrEnum` which uses lowercase names as values when using `auto()`.
     let mut auto_counter = 0;
 
-    let ignored_names: Option<Vec<&str>> = if let Some(ignore) = table.place_id_by_name("_ignore_")
-    {
-        let ignore_bindings = use_def_map.all_reachable_bindings(ignore);
+    let ignored_names: Option<Vec<&str>> = if let Some(ignore) = table.symbol_id("_ignore_") {
+        let ignore_bindings = use_def_map.all_reachable_symbol_bindings(ignore);
         let ignore_place = place_from_bindings(db, ignore_bindings);
 
         match ignore_place {
@@ -101,9 +100,9 @@ pub(crate) fn enum_metadata<'db>(
     let mut aliases = FxHashMap::default();
 
     let members = use_def_map
-        .all_end_of_scope_bindings()
-        .filter_map(|(place_id, bindings)| {
-            let name = table.place_expr(place_id).as_name()?;
+        .all_end_of_scope_symbol_bindings()
+        .filter_map(|(symbol_id, bindings)| {
+            let name = table.symbol(symbol_id).name();
 
             if name.starts_with("__") && !name.ends_with("__") {
                 // Skip private attributes
@@ -187,7 +186,7 @@ pub(crate) fn enum_metadata<'db>(
                 }
             }
 
-            let declarations = use_def_map.end_of_scope_declarations(place_id);
+            let declarations = use_def_map.end_of_scope_symbol_declarations(symbol_id);
             let declared = place_from_declarations(db, declarations);
 
             match declared {
@@ -213,9 +212,8 @@ pub(crate) fn enum_metadata<'db>(
                 }
             }
 
-            Some(name)
+            Some(name.clone())
         })
-        .cloned()
         .collect::<FxOrderSet<_>>();
 
     if members.is_empty() {
